# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v3 import credentials_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestCredentialsClient(base.BaseServiceTest):
    FAKE_CREATE_CREDENTIAL = {
        "credential": {
            "blob": "{\"access\":\"181920\",\"secret\":\"secretKey\"}",
            "project_id": "731fc6f265cd486d900f16e84c5cb594",
            "type": "ec2",
            "user_id": "bb5476fd12884539b41d5a88f838d773"
        }
    }

    FAKE_INFO_CREDENTIAL = {
        "credential": {
            "user_id": "bb5476fd12884539b41d5a88f838d773",
            "links": {
                "self": "http://example.com/identity/v3/credentials/" +
                        "207e9b76935efc03804d3dd6ab52d22e9b22a0711e4" +
                        "ada4ff8b76165a07311d7"
            },
            "blob": "{\"access\": \"a42a27755ce6442596b049bd7dd8a563\"," +
                    " \"secret\": \"71faf1d40bb24c82b479b1c6fbbd9f0c\"}",
            "project_id": "6e01855f345f4c59812999b5e459137d",
            "type": "ec2",
            "id": "207e9b76935efc03804d3dd6ab52d22e9b22a0711e4ada4f"
        }
    }

    FAKE_LIST_CREDENTIALS = {
        "credentials": [
            {
                "user_id": "bb5476fd12884539b41d5a88f838d773",
                "links": {
                    "self": "http://example.com/identity/v3/credentials/" +
                            "207e9b76935efc03804d3dd6ab52d22e9b22a0711e4" +
                            "ada4ff8b76165a07311d7"
                },
                "blob": "{\"access\": \"a42a27755ce6442596b049bd7dd8a563\"," +
                        " \"secret\": \"71faf1d40bb24c82b479b1c6fbbd9f0c\"," +
                        " \"trust_id\": null}",
                "project_id": "6e01855f345f4c59812999b5e459137d",
                "type": "ec2",
                "id": "207e9b76935efc03804d3dd6ab52d22e9b22a0711e4ada4f"
            },
            {
                "user_id": "6f556708d04b4ea6bc72d7df2296b71a",
                "links": {
                    "self": "http://example.com/identity/v3/credentials/" +
                            "2441494e52ab6d594a34d74586075cb299489bdd1e9" +
                            "389e3ab06467a4f460609"
                },
                "blob": "{\"access\": \"7da79ff0aa364e1396f067e352b9b79a\"," +
                        " \"secret\": \"7a18d68ba8834b799d396f3ff6f1e98c\"," +
                        " \"trust_id\": null}",
                "project_id": "1a1d14690f3c4ec5bf5f321c5fde3c16",
                "type": "ec2",
                "id": "2441494e52ab6d594a34d74586075cb299489bdd1e9389e3"
            },
            {
                "user_id": "c14107e65d5c4a7f8894fc4b3fc209ff",
                "links": {
                    "self": "http://example.com/identity/v3/credentials/" +
                            "3397b204b5f04c495bcdc8f34c8a39996f280f91726" +
                            "58241873e15f070ec79d7"
                },
                "blob": "{\"access\": \"db9c58a558534a10a070110de4f9f20c\"," +
                        " \"secret\": \"973e790b88db447ba6f93bca02bc745b\"," +
                        " \"trust_id\": null}",
                "project_id": "7396e43183db40dcbf40dd727637b548",
                "type": "ec2",
                "id": "3397b204b5f04c495bcdc8f34c8a39996f280f9172658241"
            },
            {
                "user_id": "bb5476fd12884539b41d5a88f838d773",
                "links": {
                    "self": "http://example.com/identity/v3/credentials/" +
                            "7ef4faa904ae7b8b4ddc7bad15b05ee359dad7d7a9b" +
                            "82861d4ad92fdbbb2eb4e"
                },
                "blob": "{\"access\": \"7d7559359b57419eb5f5f5dcd65ab57d\"," +
                        " \"secret\": \"570652bcf8c2483c86eb29e9734eed3c\"," +
                        " \"trust_id\": null}",
                "project_id": "731fc6f265cd486d900f16e84c5cb594",
                "type": "ec2",
                "id": "7ef4faa904ae7b8b4ddc7bad15b05ee359dad7d7a9b82861"
            },
        ],
        "links": {
            "self": "http://example.com/identity/v3/credentials",
            "previous": None,
            "next": None
        }
    }

    def setUp(self):
        super(TestCredentialsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = credentials_client.CredentialsClient(fake_auth,
                                                           'identity',
                                                           'regionOne')

    def _test_create_credential(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_credential,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_CREDENTIAL,
            bytes_body, status=201)

    def _test_show_credential(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_credential,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_CREDENTIAL,
            bytes_body,
            credential_id="207e9b76935efc03804d3dd6ab52d22e9b22a0711e4ada4f")

    def _test_update_credential(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_credential,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_INFO_CREDENTIAL,
            bytes_body,
            credential_id="207e9b76935efc03804d3dd6ab52d22e9b22a0711e4ada4f")

    def _test_list_credentials(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_credentials,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_CREDENTIALS,
            bytes_body)

    def test_create_credential_with_str_body(self):
        self._test_create_credential()

    def test_create_credential_with_bytes_body(self):
        self._test_create_credential(bytes_body=True)

    def test_show_credential_with_str_body(self):
        self._test_show_credential()

    def test_show_credential_with_bytes_body(self):
        self._test_show_credential(bytes_body=True)

    def test_update_credential_with_str_body(self):
        self._test_update_credential()

    def test_update_credential_with_bytes_body(self):
        self._test_update_credential(bytes_body=True)

    def test_list_credentials_with_str_body(self):
        self._test_list_credentials()

    def test_list_credentials_with_bytes_body(self):
        self._test_list_credentials(bytes_body=True)

    def test_delete_credential(self):
        self.check_service_client_function(
            self.client.delete_credential,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            credential_id="207e9b76935efc03804d3dd6ab52d22e9b22a0711e4ada4f",
            status=204)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Tempest documentation build configuration file, created by
# sphinx-quickstart on Tue May 21 17:43:32 2013.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import subprocess
import sys

# Build the plugin registry
def build_plugin_registry(app):
    root_dir = os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
    subprocess.call(['tools/generate-tempest-plugins-list.sh'], cwd=root_dir)

def autodoc_skip_member_handler(app, what, name, obj, skip, options):
    return skip or (what == "class" and not name.startswith("test"))

def setup(app):
    app.connect('autodoc-skip-member', autodoc_skip_member_handler)
    if os.getenv('GENERATE_TEMPEST_PLUGIN_LIST', 'true').lower() == 'true':
        app.connect('builder-inited', build_plugin_registry)

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
# sys.path.insert(0, os.path.abspath('.'))
sys.path.insert(0, os.path.abspath('../../tempest'))
sys.path.insert(0, os.path.abspath('../../tempest/api'))

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.todo',
              'sphinx.ext.viewcode',
              'sphinxcontrib.rsvgconverter',
              'openstackdocstheme',
              'oslo_config.sphinxconfiggen',
             ]

config_generator_config_file = '../../tempest/cmd/config-generator.tempest.conf'
sample_config_basename = '_static/tempest'

todo_include_todos = True

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/tempest'
openstackdocs_bug_project = 'tempest'
openstackdocs_bug_tag = 'doc'
openstackdocs_pdf_link = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
copyright = '2013, OpenStack QA Team'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['tempest.']


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'openstackdocs'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']
# Add any paths that contain "extra" files, such as .htaccess or
# robots.txt.
html_extra_path = ['_extra']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_domain_indices = False

# If false, no index is generated.
html_use_index = False

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# A list of warning types to suppress arbitrary warning messages.
suppress_warnings = ['image.nonlocal_uri']

# -- Options for LaTeX output -------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index', 'doc-tempest.tex', 'Tempest Testing Project',
     'OpenStack Foundation', 'manual'),
]

latex_use_xindy = False

latex_elements = {
    'maxlistdepth': 20,
    'printindex': '\\footnotesize\\raggedright\\printindex'
}



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class EndPointGroupsClient(rest_client.RestClient):
    api_version = "v3"

    def create_endpoint_group(self, **kwargs):
        """Create endpoint group.

        For a full list of available parameters, please refer to the
        official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#create-endpoint-group
        """
        post_body = json.dumps({'endpoint_group': kwargs})
        resp, body = self.post('OS-EP-FILTER/endpoint_groups', post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_endpoint_group(self, endpoint_group_id, **kwargs):
        """Update endpoint group.

        For a full list of available parameters, please refer to the
        official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#update-endpoint-group
        """
        post_body = json.dumps({'endpoint_group': kwargs})
        resp, body = self.patch(
            'OS-EP-FILTER/endpoint_groups/%s' % endpoint_group_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_endpoint_group(self, endpoint_group_id):
        """Delete endpoint group."""
        resp_header, resp_body = self.delete(
            'OS-EP-FILTER/endpoint_groups/%s' % endpoint_group_id)
        self.expected_success(204, resp_header.status)
        return rest_client.ResponseBody(resp_header, resp_body)

    def show_endpoint_group(self, endpoint_group_id):
        """Get endpoint group."""
        resp_header, resp_body = self.get(
            'OS-EP-FILTER/endpoint_groups/%s' % endpoint_group_id)
        self.expected_success(200, resp_header.status)
        resp_body = json.loads(resp_body)
        return rest_client.ResponseBody(resp_header, resp_body)

    def check_endpoint_group(self, endpoint_group_id):
        """Check endpoint group."""
        resp_header, resp_body = self.head(
            'OS-EP-FILTER/endpoint_groups/%s' % endpoint_group_id)
        self.expected_success(200, resp_header.status)
        return rest_client.ResponseBody(resp_header, resp_body)

    def list_endpoint_groups(self):
        """Get endpoint groups."""
        resp_header, resp_body = self.get('OS-EP-FILTER/endpoint_groups')
        self.expected_success(200, resp_header.status)
        resp_body = json.loads(resp_body)
        return rest_client.ResponseBody(resp_header, resp_body)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import role_assignments_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestRoleAssignmentsClient(base.BaseServiceTest):

    FAKE_USER_ID = "313234"
    FAKE_GROUP_ID = "101112"

    FAKE_ROLE1_ID = "123456"
    FAKE_ROLE2_ID = "123457"

    FAKE_PROJECT_ID = "456789"
    FAKE_DOMAIN_ID = "102030"

    FAKE_USER_PROJECT_ASSIGNMENT = {
        "links": {
            "assignment": "http://example.com/identity/v3/projects/"
                          "%s/users/%s/roles/%s" % (FAKE_PROJECT_ID,
                                                    FAKE_USER_ID,
                                                    FAKE_ROLE2_ID)
        },
        "role": {
            "id": FAKE_ROLE2_ID
        },
        "scope": {
            "project": {
                "id": FAKE_PROJECT_ID
            }
        },
        "user": {
            "id": FAKE_USER_ID
        }
    }

    FAKE_GROUP_PROJECT_ASSIGNMENT = {
        "links": {
            "assignment": "http://example.com/identity/v3/projects/"
                          "%s/groups/%s/roles/%s" % (FAKE_PROJECT_ID,
                                                     FAKE_GROUP_ID,
                                                     FAKE_ROLE1_ID)
        },
        "role": {
            "id": FAKE_ROLE1_ID
        },
        "scope": {
            "project": {
                "id": FAKE_PROJECT_ID
            }
        },
        "group": {
            "id": FAKE_GROUP_ID
        }
    }

    FAKE_USER_PROJECT_EFFECTIVE_ASSIGNMENT = {
        "links": {
            "assignment": "http://example.com/identity/v3/projects/"
                          "%s/groups/%s/roles/%s" % (FAKE_PROJECT_ID,
                                                     FAKE_GROUP_ID,
                                                     FAKE_ROLE1_ID),
            "membership": "http://example.com/identity/v3/groups/"
                          "%s/users/%s" % (FAKE_GROUP_ID, FAKE_USER_ID)
        },
        "role": {
            "id": FAKE_ROLE1_ID
        },
        "scope": {
            "project": {
                "id": FAKE_PROJECT_ID
            }
        },
        "user": {
            "id": FAKE_USER_ID
        }
    }

    FAKE_USER_DOMAIN_ASSIGNMENT = {
        "links": {
            "assignment": "http://example.com/identity/v3/domains/"
                          "%s/users/%s/roles/%s" % (FAKE_DOMAIN_ID,
                                                    FAKE_USER_ID,
                                                    FAKE_ROLE1_ID)
        },
        "role": {
            "id": FAKE_ROLE1_ID
        },
        "scope": {
            "domain": {
                "id": FAKE_DOMAIN_ID
            }
        },
        "user": {
            "id": FAKE_USER_ID
        }
    }

    FAKE_GROUP_PROJECT_ASSIGNMENTS = {
        "role_assignments": [
            FAKE_GROUP_PROJECT_ASSIGNMENT
        ],
        "links": {
            "self": "http://example.com/identity/v3/role_assignments?"
                    "scope.project.id=%s&group.id=%s&effective" % (
                        FAKE_PROJECT_ID, FAKE_GROUP_ID),
            "previous": None,
            "next": None
        }
    }

    FAKE_USER_PROJECT_EFFECTIVE_ASSIGNMENTS = {
        "role_assignments": [
            FAKE_USER_PROJECT_ASSIGNMENT,
            FAKE_USER_PROJECT_EFFECTIVE_ASSIGNMENT
        ],
        "links": {
            "self": "http://example.com/identity/v3/role_assignments?"
                    "scope.project.id=%s&user.id=%s&effective" % (
                        FAKE_PROJECT_ID, FAKE_USER_ID),
            "previous": None,
            "next": None
        }
    }

    FAKE_USER_DOMAIN_ASSIGNMENTS = {
        "role_assignments": [
            FAKE_USER_DOMAIN_ASSIGNMENT
        ],
        "links": {
            "self": "http://example.com/identity/v3/role_assignments?"
                    "scope.domain.id=%s&user.id=%s&effective" % (
                        FAKE_DOMAIN_ID, FAKE_USER_ID),
            "previous": None,
            "next": None
        }
    }

    def setUp(self):
        super(TestRoleAssignmentsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = role_assignments_client.RoleAssignmentsClient(
            fake_auth, 'identity', 'regionOne')

    def _test_list_user_project_effective_assignments(self, bytes_body=False):
        params = {'scope.project.id': self.FAKE_PROJECT_ID,
                  'user.id': self.FAKE_USER_ID}
        self.check_service_client_function(
            self.client.list_role_assignments,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_USER_PROJECT_EFFECTIVE_ASSIGNMENTS,
            bytes_body,
            effective=True,
            **params)

    def test_list_user_project_effective_assignments_with_str_body(self):
        self._test_list_user_project_effective_assignments()

    def test_list_user_project_effective_assignments_with_bytes_body(self):
        self._test_list_user_project_effective_assignments(bytes_body=True)

    def _test_list_group_project_assignments(self, bytes_body=False):
        params = {'scope.project.id': self.FAKE_PROJECT_ID,
                  'group.id': self.FAKE_GROUP_ID}
        self.check_service_client_function(
            self.client.list_role_assignments,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_GROUP_PROJECT_ASSIGNMENTS,
            bytes_body,
            **params)

    def test_list_group_project_assignments_with_str_body(self):
        self._test_list_group_project_assignments()

    def test_list_group_project_assignments_with_bytes_body(self):
        self._test_list_group_project_assignments(bytes_body=True)

    def _test_list_user_domain_assignments(self, bytes_body=False):
        params = {'scope.domain.id': self.FAKE_DOMAIN_ID,
                  'user.id': self.FAKE_USER_ID}
        self.check_service_client_function(
            self.client.list_role_assignments,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_GROUP_PROJECT_ASSIGNMENTS,
            bytes_body,
            **params)

    def test_list_user_domain_assignments_with_str_body(self):
        self._test_list_user_domain_assignments()

    def test_list_user_domain_assignments_with_bytes_body(self):
        self._test_list_user_domain_assignments(bytes_body=True)



# Copyright 2016 Hewlett Packard Enterprise Development Company
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import netaddr

from tempest.lib import exceptions as lib_exc


def get_unused_ip_addresses(ports_client, subnets_client,
                            network_id, subnet_id, count):
    """Return a list with the specified number of unused IP addresses

    This method uses the given ports_client to find the specified number of
    unused IP addresses on the given subnet using the supplied subnets_client
    """

    ports = ports_client.list_ports(network_id=network_id)['ports']
    subnet = subnets_client.show_subnet(subnet_id)
    ip_net = netaddr.IPNetwork(subnet['subnet']['cidr'])
    subnet_set = netaddr.IPSet(ip_net.iter_hosts())
    alloc_set = netaddr.IPSet()

    # prune out any addresses already allocated to existing ports
    for port in ports:
        for fixed_ip in port.get('fixed_ips'):
            alloc_set.add(fixed_ip['ip_address'])

    # exclude gateway_ip of subnet
    gateway_ip = subnet['subnet']['gateway_ip']
    if gateway_ip:
        alloc_set.add(gateway_ip)

    av_set = subnet_set - alloc_set
    addrs = []
    for cidr in reversed(av_set.iter_cidrs()):
        for ip in reversed(cidr):
            addrs.append(str(ip))
            if len(addrs) == count:
                return addrs
    msg = "Insufficient IP addresses available"
    raise lib_exc.BadRequest(message=msg)


def get_ping_payload_size(mtu, ip_version):
    """Return the maximum size of ping payload that will fit into MTU."""
    if not mtu:
        return None
    if ip_version == 4:
        ip_header = 20
        icmp_header = 8
    else:
        ip_header = 40
        icmp_header = 4
    res = mtu - ip_header - icmp_header
    if res < 0:
        raise lib_exc.BadRequest(
            message='MTU = %(mtu)d is too low for IPv%(ip_version)d' % {
                'mtu': mtu,
                'ip_version': ip_version,
            })
    return res



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

CONF = config.CONF


class MigrationsAdminTest(base.BaseV2ComputeAdminTest):
    """Test migration operations supported by admin user"""

    @classmethod
    def setup_clients(cls):
        super(MigrationsAdminTest, cls).setup_clients()
        cls.client = cls.os_admin.migrations_client

    @decorators.idempotent_id('75c0b83d-72a0-4cf8-a153-631e83e7d53f')
    def test_list_migrations(self):
        """Test admin user can get the migrations list"""
        self.client.list_migrations()

    @decorators.idempotent_id('1b512062-8093-438e-b47a-37d2f597cd64')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_list_migrations_in_flavor_resize_situation(self):
        """Admin can get the migrations list containing the resized server"""
        server = self.create_test_server(wait_until="ACTIVE")
        server_id = server['id']

        self.resize_server(server_id, self.flavor_ref_alt)

        body = self.client.list_migrations()['migrations']

        instance_uuids = [x['instance_uuid'] for x in body]
        self.assertIn(server_id, instance_uuids)

    def _flavor_clean_up(self, flavor_id):
        try:
            self.admin_flavors_client.delete_flavor(flavor_id)
            self.admin_flavors_client.wait_for_resource_deletion(flavor_id)
        except exceptions.NotFound:
            pass

    @decorators.idempotent_id('33f1fec3-ba18-4470-8e4e-1d888e7c3593')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_revert_deleted_flavor(self):
        """Test reverting resized server with original flavor deleted

        Tests that we can revert the resize on an instance whose original
        flavor has been deleted.
        """

        # First we have to create a flavor that we can delete so make a copy
        # of the normal flavor from which we'd create a server.
        flavor = self.admin_flavors_client.show_flavor(
            self.flavor_ref)['flavor']
        flavor = self.admin_flavors_client.create_flavor(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='test_resize_flavor_'),
            ram=flavor['ram'],
            disk=flavor['disk'],
            vcpus=flavor['vcpus']
        )['flavor']
        self.addCleanup(self._flavor_clean_up, flavor['id'])

        # Set extra specs same as self.flavor_ref for the created flavor,
        # because the environment may need some special extra specs to
        # create server which should have been contained in
        # self.flavor_ref.
        extra_spec_keys = self.admin_flavors_client.list_flavor_extra_specs(
            self.flavor_ref)['extra_specs']
        if extra_spec_keys:
            self.admin_flavors_client.set_flavor_extra_spec(
                flavor['id'], **extra_spec_keys)

        # Now boot a server with the copied flavor.
        server = self.create_test_server(
            wait_until='ACTIVE', flavor=flavor['id'])
        server = self.servers_client.show_server(server['id'])['server']

        # If 'id' not in server['flavor'], we can only compare the flavor
        # details, so here we should save the to-be-deleted flavor's details,
        # for the flavor comparison after the server resizing.
        if not server['flavor'].get('id'):
            pre_flavor = {}
            body = self.flavors_client.show_flavor(flavor['id'])['flavor']
            for key in ['name', 'ram', 'vcpus', 'disk']:
                pre_flavor[key] = body[key]

        # Delete the flavor we used to boot the instance.
        self._flavor_clean_up(flavor['id'])

        # Now resize the server and wait for it to go into verify state.
        self.servers_client.resize_server(server['id'], self.flavor_ref_alt)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'VERIFY_RESIZE')

        # Now revert the resize, it should be OK even though the original
        # flavor used to boot the server was deleted.
        self.servers_client.revert_resize_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

        server = self.servers_client.show_server(server['id'])['server']
        if server['flavor'].get('id'):
            msg = ('server flavor is not same as flavor!')
            self.assertEqual(flavor['id'], server['flavor']['id'], msg)
        else:
            self.assertEqual(pre_flavor['name'],
                             server['flavor']['original_name'],
                             "original_name in server flavor is not same as "
                             "flavor name!")
            for key in ['ram', 'vcpus', 'disk']:
                msg = ('attribute %s in server flavor is not same as '
                       'flavor!' % key)
                self.assertEqual(pre_flavor[key], server['flavor'][key], msg)

    def _test_cold_migrate_server(self, revert=False):
        if CONF.compute.min_compute_nodes < 2:
            msg = "Less than 2 compute nodes, skipping multinode tests."
            raise self.skipException(msg)

        server = self.create_test_server(wait_until="ACTIVE")
        src_host = self.get_host_for_server(server['id'])

        self.admin_servers_client.migrate_server(server['id'])

        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'VERIFY_RESIZE')

        if revert:
            self.servers_client.revert_resize_server(server['id'])
            assert_func = self.assertEqual
        else:
            self.servers_client.confirm_resize_server(server['id'])
            assert_func = self.assertNotEqual

        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')
        dst_host = self.get_host_for_server(server['id'])
        assert_func(src_host, dst_host)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('4bf0be52-3b6f-4746-9a27-3143636fe30d')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration not available.')
    def test_cold_migration(self):
        """Test cold migrating server and then confirm the migration"""
        self._test_cold_migrate_server(revert=False)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('caa1aa8b-f4ef-4374-be0d-95f001c2ac2d')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration not available.')
    def test_revert_cold_migration(self):
        """Test cold migrating server and then revert the migration"""
        self._test_cold_migrate_server(revert=True)



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume.v3_61 import volumes as schemav361
from tempest.lib.api_schema.response.volume.v3_63 import volumes as schemav363
from tempest.lib.api_schema.response.volume.v3_64 import volumes as schemav364
from tempest.lib.api_schema.response.volume.v3_65 import volumes as schemav365
from tempest.lib.api_schema.response.volume.v3_69 import volumes as schemav369
from tempest.lib.api_schema.response.volume import volumes as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.volume import base_client


class VolumesClient(base_client.BaseClient):
    """Client class to send CRUD Volume V3 API requests"""

    schema_versions_info = [
        {'min': None, 'max': '3.60', 'schema': schema},
        {'min': '3.61', 'max': '3.62', 'schema': schemav361},
        {'min': '3.63', 'max': '3.63', 'schema': schemav363},
        {'min': '3.64', 'max': '3.64', 'schema': schemav364},
        {'min': '3.65', 'max': '3.68', 'schema': schemav365},
        {'min': '3.69', 'max': None, 'schema': schemav369}
        ]

    def _prepare_params(self, params):
        """Prepares params for use in get or _ext_get methods.

        If params is a string it will be left as it is, but if it's not it will
        be urlencoded.
        """
        if isinstance(params, str):
            return params
        return urllib.urlencode(params)

    def list_hosts(self):
        """Lists all hosts summary info that is not disabled.

        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-all-hosts-for-a-project
        """
        resp, body = self.get('os-hosts')
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_volumes(self, detail=False, params=None):
        """List all the volumes created.

        Params can be a string (must be urlencoded) or a dictionary.
        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-accessible-volumes-with-details
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#list-accessible-volumes
        """
        url = 'volumes'
        schema = self.get_schema(self.schema_versions_info)
        list_schema = schema.list_volumes_no_detail
        if detail:
            list_schema = schema.list_volumes_with_detail
            url += '/detail'
        if params:
            url += '?%s' % self._prepare_params(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(list_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def migrate_volume(self, volume_id, **kwargs):
        """Migrate a volume to a new backend

        For a full list of available parameters please refer to the official
        API reference:

        https://docs.openstack.org/api-ref/block-storage/v3/index.html#migrate-a-volume
        """
        post_body = json.dumps({'os-migrate_volume': kwargs})
        resp, body = self.post('volumes/%s/action' % volume_id, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_volume(self, volume_id):
        """Returns the details of a single volume."""
        url = "volumes/%s" % volume_id
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_volume(self, **kwargs):
        """Creates a new Volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#create-a-volume
        """
        post_body = json.dumps({'volume': kwargs})
        resp, body = self.post('volumes', post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_volume(self, volume_id, **kwargs):
        """Updates the Specified Volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#update-a-volume
        """
        put_body = json.dumps({'volume': kwargs})
        resp, body = self.put('volumes/%s' % volume_id, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_volume(self, volume_id, **params):
        """Deletes the Specified Volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#delete-a-volume
        """
        url = 'volumes/%s' % volume_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.delete(url)
        self.validate_response(schema.delete_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_summary(self, **params):
        """Get volumes summary.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#get-volumes-summary
        """
        url = 'volumes/summary'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_volume_summary, resp, body)
        return rest_client.ResponseBody(resp, body)

    def upload_volume(self, volume_id, **kwargs):
        """Uploads a volume in Glance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#upload-volume-to-image
        """
        post_body = json.dumps({'os-volume_upload_image': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        # TODO(zhufl): This is under discussion, so will be merged
        # in a separate patch.
        # https://bugs.launchpad.net/cinder/+bug/1880566
        # self.validate_response(schema.upload_volume, resp, body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def attach_volume(self, volume_id, **kwargs):
        """Attaches a volume to a given instance on a given mountpoint.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#attach-volume-to-a-server
        """
        post_body = json.dumps({'os-attach': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.attach_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_bootable_volume(self, volume_id, **kwargs):
        """Set a bootable flag for a volume - true or false.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#update-a-volume-s-bootable-status
        """
        post_body = json.dumps({'os-set_bootable': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.set_bootable_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def detach_volume(self, volume_id, **kwargs):
        """Detaches a volume from an instance."""
        post_body = json.dumps({'os-detach': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.detach_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def terminate_connection(self, volume_id, connector):
        """Detaches a volume from an instance using terminate_connection."""
        post_body = json.dumps(
            {'os-terminate_connection': {'connector': connector}})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.terminate_connection, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reserve_volume(self, volume_id):
        """Reserves a volume."""
        post_body = json.dumps({'os-reserve': {}})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.reserve_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def unreserve_volume(self, volume_id):
        """Restore a reserved volume ."""
        post_body = json.dumps({'os-unreserve': {}})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.unreserve_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        """Check the specified resource is deleted or not.

        :param id: A checked resource id
        :raises lib_exc.DeleteErrorException: If the specified resource is on
            the status the delete was failed.
        """
        try:
            volume = self.show_volume(id)
        except lib_exc.NotFound:
            return True
        if volume["volume"]["status"] == "error_deleting":
            raise lib_exc.DeleteErrorException(
                "Volume %s failed to delete and is in error_deleting status" %
                volume['volume']['id'])
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'volume'

    def extend_volume(self, volume_id, **kwargs):
        """Extend a volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#extend-a-volume-size
        """
        post_body = json.dumps({'os-extend': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.extend_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reset_volume_status(self, volume_id, **kwargs):
        """Reset the Specified Volume's Status.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#reset-a-volume-s-statuses
        """
        post_body = json.dumps({'os-reset_status': kwargs})
        resp, body = self.post('volumes/%s/action' % volume_id, post_body)
        self.validate_response(schema.reset_volume_status, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_volume_readonly(self, volume_id, **kwargs):
        """Update the Specified Volume readonly.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#updates-volume-read-only-access-mode-flag
        """
        post_body = json.dumps({'os-update_readonly_flag': kwargs})
        url = 'volumes/%s/action' % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.update_volume_readonly, resp, body)
        return rest_client.ResponseBody(resp, body)

    def force_delete_volume(self, volume_id):
        """Force Delete Volume."""
        post_body = json.dumps({'os-force_delete': {}})
        resp, body = self.post('volumes/%s/action' % volume_id, post_body)
        self.validate_response(schema.force_delete_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_volume_metadata(self, volume_id, metadata):
        """Create metadata for the volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#create-metadata-for-volume
        """
        put_body = json.dumps({'metadata': metadata})
        url = "volumes/%s/metadata" % volume_id
        resp, body = self.post(url, put_body)
        body = json.loads(body)
        self.validate_response(schema.create_volume_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_metadata(self, volume_id):
        """Get metadata of the volume."""
        url = "volumes/%s/metadata" % volume_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_volume_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_volume_metadata(self, volume_id, metadata):
        """Update metadata for the volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#update-a-volume-s-metadata
        """
        put_body = json.dumps({'metadata': metadata})
        url = "volumes/%s/metadata" % volume_id
        resp, body = self.put(url, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_volume_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_metadata_item(self, volume_id, id):
        """Show metadata item for the volume."""
        url = "volumes/%s/metadata/%s" % (volume_id, id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_volume_metadata_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_volume_metadata_item(self, volume_id, id, meta_item):
        """Update metadata item for the volume."""
        put_body = json.dumps({'meta': meta_item})
        url = "volumes/%s/metadata/%s" % (volume_id, id)
        resp, body = self.put(url, put_body)
        body = json.loads(body)
        self.validate_response(schema.update_volume_metadata_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_volume_metadata_item(self, volume_id, id):
        """Delete metadata item for the volume."""
        url = "volumes/%s/metadata/%s" % (volume_id, id)
        resp, body = self.delete(url)
        self.validate_response(schema.delete_volume_metadata_item, resp, body)
        return rest_client.ResponseBody(resp, body)

    def retype_volume(self, volume_id, **kwargs):
        """Updates volume with new volume type.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#retype-a-volume
        """
        post_body = json.dumps({'os-retype': kwargs})
        resp, body = self.post('volumes/%s/action' % volume_id, post_body)
        self.validate_response(schema.retype_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def force_detach_volume(self, volume_id, **kwargs):
        """Force detach a volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#force-detach-a-volume
        """
        post_body = json.dumps({'os-force_detach': kwargs})
        url = 'volumes/%s/action' % volume_id
        resp, body = self.post(url, post_body)
        self.validate_response(schema.force_detach_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_volume_image_metadata(self, volume_id, **kwargs):
        """Update image metadata for the volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#set-image-metadata-for-a-volume
        """
        post_body = json.dumps({'os-set_image_metadata': {'metadata': kwargs}})
        url = "volumes/%s/action" % (volume_id)
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(schema.update_volume_image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_volume_image_metadata(self, volume_id, key_name):
        """Delete image metadata item for the volume."""
        post_body = json.dumps({'os-unset_image_metadata': {'key': key_name}})
        url = "volumes/%s/action" % (volume_id)
        resp, body = self.post(url, post_body)
        self.validate_response(schema.delete_volume_image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_image_metadata(self, volume_id):
        """Show image metadata for the volume."""
        post_body = json.dumps({'os-show_image_metadata': {}})
        url = "volumes/%s/action" % volume_id
        resp, body = self.post(url, post_body)
        body = json.loads(body)
        self.validate_response(schema.show_volume_image_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def unmanage_volume(self, volume_id):
        """Unmanage volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/index.html#unmanage-a-volume
        """
        post_body = json.dumps({'os-unmanage': {}})
        resp, body = self.post('volumes/%s/action' % volume_id, post_body)
        self.validate_response(schema.unmanage_volume, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.api.network import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ExternalNetworksAdminNegativeTestJSON(base.BaseAdminNetworkTest):
    """Negative tests of external network"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d402ae6c-0be0-4d8e-833b-a738895d98d0')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_create_port_with_precreated_floatingip_as_fixed_ip(self):
        """Test creating port with precreated floating ip as fixed ip

        NOTE: External networks can be used to create both floating-ip as
        well as instance-ip. So, creating an instance-ip with a value of a
        pre-created floating-ip should be denied.
        """
        # create a floating ip
        body = self.admin_floating_ips_client.create_floatingip(
            floating_network_id=CONF.network.public_network_id)
        created_floating_ip = body['floatingip']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_floating_ips_client.delete_floatingip,
                        created_floating_ip['id'])
        floating_ip_address = created_floating_ip['floating_ip_address']
        self.assertIsNotNone(floating_ip_address)

        # use the same value of floatingip as fixed-ip to create_port()
        fixed_ips = [{'ip_address': floating_ip_address}]

        # create a port which will internally create an instance-ip
        self.assertRaises(lib_exc.Conflict,
                          self.admin_ports_client.create_port,
                          name=data_utils.rand_name(
                              self.__class__.__name__,
                              prefix=CONF.resource_name_prefix),
                          network_id=CONF.network.public_network_id,
                          fixed_ips=fixed_ips)



# Copyright 2017 AT&T Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class DomainConfigurationClient(rest_client.RestClient):
    api_version = "v3"

    def show_default_config_settings(self):
        """Show default configuration settings.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-default-configuration-settings
        """
        url = 'domains/config/default'
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_default_group_config(self, group):
        """Show default configuration for a group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-default-configuration-for-a-group
        """
        url = 'domains/config/%s/default' % group
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_default_group_option(self, group, option):
        """Show default option for a group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-default-option-for-a-group
        """
        url = 'domains/config/%s/%s/default' % (group, option)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_domain_group_option_config(self, domain_id, group, option):
        """Show domain group option configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-domain-group-option-configuration
        """
        url = 'domains/%s/config/%s/%s' % (domain_id, group, option)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_domain_group_option_config(self, domain_id, group, option,
                                          **kwargs):
        """Update domain group option configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-domain-group-option-configuration
        """
        url = 'domains/%s/config/%s/%s' % (domain_id, group, option)
        resp, body = self.patch(url, json.dumps({'config': kwargs}))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_domain_group_option_config(self, domain_id, group, option):
        """Delete domain group option configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#delete-domain-group-option-configuration
        """
        url = 'domains/%s/config/%s/%s' % (domain_id, group, option)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_domain_group_config(self, domain_id, group):
        """Shows details for a domain group configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-domain-group-configuration
        """
        url = 'domains/%s/config/%s' % (domain_id, group)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_domain_group_config(self, domain_id, group, **kwargs):
        """Update domain group configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-domain-group-configuration
        """
        url = 'domains/%s/config/%s' % (domain_id, group)
        resp, body = self.patch(url, json.dumps({'config': kwargs}))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_domain_group_config(self, domain_id, group):
        """Delete domain group configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#delete-domain-group-configuration
        """
        url = 'domains/%s/config/%s' % (domain_id, group)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_domain_config(self, domain_id, **kwargs):
        """Create domain configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-domain-configuration
        """
        url = 'domains/%s/config' % domain_id
        resp, body = self.put(url, json.dumps({'config': kwargs}))
        self.expected_success([200, 201], resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_domain_config(self, domain_id):
        """Show domain configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-domain-configuration
        """
        url = 'domains/%s/config' % domain_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_domain_config(self, domain_id, **kwargs):
        """Update domain configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-domain-configuration
        """
        url = 'domains/%s/config' % domain_id
        resp, body = self.patch(url, json.dumps({'config': kwargs}))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_domain_config(self, domain_id):
        """Delete domain configuration.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#delete-domain-configuration
        """
        url = 'domains/%s/config' % domain_id
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import groups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestGroupsClient(base.BaseServiceTest):
    FAKE_CREATE_GROUP = {
        "group": {
            "id": "6f519a48-3183-46cf-a32f-41815f816666",
            "name": "first_group"
        }
    }

    FAKE_CREATE_GROUP_FROM_GROUP_SNAPSHOT = {
        "group": {
            "id": "6f519a48-3183-46cf-a32f-41815f816668",
            "name": "first_group"
        }
    }

    FAKE_CREATE_GROUP_FROM_GROUP = {
        "group": {
            "id": "6f519a48-3183-46cf-a32f-41815f816667",
            "name": "other_group"
        }
    }

    FAKE_UPDATE_GROUP = {
        "group": {
            "name": "new-group",
            "description": "New test group",
            "add_volumes": "27d45037-ade3-4a87-b729-dba3293c06f3,"
                           "6e7cd916-d961-41cc-b3bd-0601ca0c701f",
            "remove_volumes": "4d580519-6467-448e-95e9-5b25c94d83c7,"
                              "ea22464c-f095-4a87-a31f-c5d34e0c6fc9"
        }
    }

    FAKE_INFO_GROUP = {
        "group": {
            "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
            "name": "group-001",
            "description": "Test group 1",
            "group_type": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
            "volume_types": ["2103099d-7cc3-4e52-a2f1-23a5284416f3"],
            "status": "available",
            "availability_zone": "az1",
            "created_at": "2017-06-20T03:50:07Z"
        }
    }

    FAKE_LIST_GROUP_DETAILS = {
        "groups": [
            {
                "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
                "name": "group-001",
                "description": "Test group 1",
                "group_type": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "volume_types": ["2103099d-7cc3-4e52-a2f1-23a5284416f3"],
                "status": "available",
                "availability_zone": "az1",
                "created_at": "2017-06-20T03:50:07Z",
            },
            {
                "id": "e479997c-650b-40a4-9dfe-77655818b0d2",
                "name": "group-002",
                "description": "Test group 2",
                "group_snapshot_id": "79c9afdb-7e46-4d71-9249-1f022886963c",
                "group_type": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "volume_types": ["2103099d-7cc3-4e52-a2f1-23a5284416f3"],
                "status": "available",
                "availability_zone": "az1",
                "created_at": "2017-06-19T01:52:47Z",
            },
            {
                "id": "c5c4769e-213c-40a6-a568-8e797bb691d4",
                "name": "group-003",
                "description": "Test group 3",
                "source_group_id": "e92f9dc7-0b20-492d-8ab2-3ad8fdac270e",
                "group_type": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "volume_types": ["2103099d-7cc3-4e52-a2f1-23a5284416f3"],
                "status": "available",
                "availability_zone": "az1",
                "created_at": "2017-06-18T06:34:32Z",
            }
        ]
    }

    FAKE_LIST_GROUPS = {
        "groups": [
            {
                "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
                "name": "group-001",
            },
            {
                "id": "e479997c-650b-40a4-9dfe-77655818b0d2",
                "name": "group-002",
            }
        ]
    }

    def setUp(self):
        super(TestGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = groups_client.GroupsClient(fake_auth,
                                                 'volume',
                                                 'regionOne')

    def _test_create_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP,
            bytes_body,
            status=202)

    def _test_show_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_GROUP,
            bytes_body,
            group_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_list_group_details(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_GROUP_DETAILS,
            bytes_body,
            detail=True)

    def _test_list_groups(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_GROUPS,
            bytes_body,
            detail=False)

    def test_create_group_with_str_body(self):
        self._test_create_group()

    def test_create_group_with_bytes_body(self):
        self._test_create_group(bytes_body=True)

    def test_show_group_with_str_body(self):
        self._test_show_group()

    def test_show_group_with_bytes_body(self):
        self._test_show_group(bytes_body=True)

    def test_list_group_details_with_str_body(self):
        self._test_list_group_details()

    def test_list_group_details_with_bytes_body(self):
        self._test_list_group_details(bytes_body=True)

    def test_list_groups_with_str_body(self):
        self._test_list_groups()

    def test_list_groups_with_bytes_body(self):
        self._test_list_groups(bytes_body=True)

    def test_delete_group(self):
        self.check_service_client_function(
            self.client.delete_group,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            group_id='0e701ab8-1bec-4b9f-b026-a7ba4af13578',
            status=202)

    def test_create_group_from_group_snapshot(self):
        self.check_service_client_function(
            self.client.create_group_from_source,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP_FROM_GROUP_SNAPSHOT,
            status=202)

    def test_create_group_from_group(self):
        self.check_service_client_function(
            self.client.create_group_from_source,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP_FROM_GROUP,
            status=202)

    def test_update_group(self):
        self.check_service_client_function(
            self.client.update_group,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            group_id='0e701ab8-1bec-4b9f-b026-a7ba4af13578',
            status=202,
            **self.FAKE_UPDATE_GROUP['group'])

    def test_reset_group_status(self):
        self.check_service_client_function(
            self.client.reset_group_status,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            group_id='0e701ab8-1bec-4b9f-b026-a7ba4af13578',
            status_to_set='error')



# Copyright 2024 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from oslo_log import log as logging

from tempest.api.compute import base as compute_base
from tempest.api.image import base as image_base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ImageDependencyTests(image_base.BaseV2ImageTest,
                           compute_base.BaseV2ComputeTest,
                           manager.ScenarioTest):
    """Test image, instance, and snapshot dependency.

       The tests create image and remove the base image that other snapshots
       were depend on.In OpenStack, images and snapshots should be separate,
       but in some configurations like Glance with Ceph storage,
       there were cases where images couldn't be removed.
       This was fixed in glance store for RBD backend.

       * Dependency scenarios:
           - image > instance -> snapshot dependency

       NOTE: volume -> image dependencies tests are in cinder-tempest-plugin
    """

    @classmethod
    def skip_checks(cls):
        super(ImageDependencyTests, cls).skip_checks()
        if not CONF.volume_feature_enabled.enable_volume_image_dep_tests:
            skip_msg = (
                "%s Volume/image dependency tests "
                "not enabled" % (cls.__name__))
            raise cls.skipException(skip_msg)

    def _create_instance_snapshot(self, bfv=False):
        """Create instance from image and then snapshot the instance."""
        # Create image and store data to image
        source = 'volume' if bfv else 'image'
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='image-dependency-test')
        image = self.create_image(name=image_name,
                                  container_format='bare',
                                  disk_format='raw',
                                  visibility='private')
        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.store_image_file(image['id'], image_file)
        waiters.wait_for_image_status(
            self.client, image['id'], 'active')
        if bfv:
            # Create instance
            instance = self.create_test_server(
                name='instance-depend-image',
                image_id=image['id'],
                volume_backed=True,
                wait_until='ACTIVE')
        else:
            # Create instance
            instance = self.create_test_server(
                name='instance-depend-image',
                image_id=image['id'],
                wait_until='ACTIVE')
        LOG.info("Instance from %s is created %s", source, instance)
        instance_observed = \
            self.servers_client.show_server(instance['id'])['server']
        # Create instance snapshot
        snapshot_instance = self.create_server_snapshot(
            server=instance_observed)
        LOG.info("Instance snapshot is created %s", snapshot_instance)
        return image['id'], snapshot_instance['id']

    @decorators.idempotent_id('d19b0731-e98e-4103-8b0e-02f651b8f586')
    @utils.services('compute')
    def test_nova_image_snapshot_dependency(self):
        """Test with image > instance > snapshot dependency.

        Create instance snapshot and check if we able to delete base
        image

        """
        base_image_id, snapshot_image_id = self._create_instance_snapshot()
        self.client.delete_image(base_image_id)
        self.client.wait_for_resource_deletion(base_image_id)
        images_list = self.client.list_images()['images']
        fetched_images_id = [img['id'] for img in images_list]
        self.assertNotIn(base_image_id, fetched_images_id)
        self.assertIn(snapshot_image_id, fetched_images_id)

    @utils.services('compute', 'volume')
    @decorators.idempotent_id('f0c8a35d-8f8f-443c-8bcb-85a9c0f87d19')
    def test_image_volume_server_snapshot_dependency(self):
        """Test with image > volume > instance > snapshot dependency.

        We are going to perform the following steps in the test:
        * Create image
        * Create a bootable volume from Image
        * Launch an instance from the bootable volume
        * Take snapshot of the instance -- which creates the volume snapshot
        * Delete the image.

        This will test the dependency chain of image -> volume -> snapshot.
        """
        base_image_id, snapshot_image_id = self._create_instance_snapshot(
            bfv=True)
        self.client.delete_image(base_image_id)
        self.client.wait_for_resource_deletion(base_image_id)
        images_list = self.client.list_images()['images']
        fetched_images_id = [img['id'] for img in images_list]
        self.assertNotIn(base_image_id, fetched_images_id)
        self.assertIn(snapshot_image_id, fetched_images_id)



# Copyright 2014 Scality
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import compute
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF


class TestShelveInstance(manager.ScenarioTest):
    """This test shelves then unshelves a Nova instance

    The following is the scenario outline:
     * boot an instance and create a timestamp file in it
     * shelve the instance
     * unshelve the instance
     * check the existence of the timestamp file in the unshelved instance
     * check the existence of the timestamp file in the unshelved instance,
       after a cold migrate

    """

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(TestShelveInstance, cls).setup_clients()
        cls.admin_servers_client = cls.os_admin.servers_client

    @classmethod
    def skip_checks(cls):
        super(TestShelveInstance, cls).skip_checks()
        if not CONF.compute_feature_enabled.shelve:
            raise cls.skipException("Shelve is not available.")

    def _shelve_then_unshelve_server(self, server):
        compute.shelve_server(self.servers_client, server['id'],
                              force_shelve_offload=True)

        self.servers_client.unshelve_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

    def _cold_migrate_server(self, server):
        src_host = self.get_host_for_server(server['id'])

        self.admin_servers_client.migrate_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'VERIFY_RESIZE')
        self.servers_client.confirm_resize_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

        dst_host = self.get_host_for_server(server['id'])
        self.assertNotEqual(src_host, dst_host)

    def _create_server_then_shelve_and_unshelve(self, boot_from_volume=False,
                                                cold_migrate=False):
        keypair = self.create_keypair()

        security_group = self.create_security_group()
        security_groups = [{'name': security_group['name']}]

        server = self.create_server(
            key_name=keypair['name'],
            security_groups=security_groups,
            volume_backed=boot_from_volume)

        instance_ip = self.get_server_ip(server)
        timestamp = self.create_timestamp(instance_ip,
                                          private_key=keypair['private_key'],
                                          server=server)

        # Prevent bug #1257594 from coming back
        # Unshelve used to boot the instance with the original image, not
        # with the instance snapshot
        self._shelve_then_unshelve_server(server)

        if cold_migrate:
            # Prevent bug #1732428 from coming back
            self._cold_migrate_server(server)

        timestamp2 = self.get_timestamp(instance_ip,
                                        private_key=keypair['private_key'],
                                        server=server)
        self.assertEqual(timestamp, timestamp2)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('1164e700-0af0-4a4c-8792-35909a88743c')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @utils.services('compute', 'network', 'image')
    def test_shelve_instance(self):
        self._create_server_then_shelve_and_unshelve()

    @decorators.attr(type='slow')
    @decorators.idempotent_id('c1b6318c-b9da-490b-9c67-9339b627271f')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @utils.services('compute', 'volume', 'network', 'image')
    def test_shelve_volume_backed_instance(self):
        self._create_server_then_shelve_and_unshelve(boot_from_volume=True)

    @decorators.attr(type=['slow', 'multinode'])
    @decorators.idempotent_id('1295fd9e-193a-4cf8-b211-55358e021bae')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration not available.')
    @testtools.skipUnless(CONF.compute_feature_enabled.shelve_migrate,
                          'Shelve migrate not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @utils.services('compute', 'network', 'image')
    def test_cold_migrate_unshelved_instance(self):
        self._create_server_then_shelve_and_unshelve(cold_migrate=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import re

from oslo_utils import timeutils

from tempest.api.identity import base
from tempest import clients
from tempest.common import credentials_factory as common_creds
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class TrustsV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone trusts"""

    @classmethod
    def skip_checks(cls):
        super(TrustsV3TestJSON, cls).skip_checks()
        if not CONF.identity_feature_enabled.trust:
            raise cls.skipException("Trusts aren't enabled")
        if CONF.identity_feature_enabled.immutable_user_source:
            raise cls.skipException('Skipped because environment has an '
                                    'immutable user source and solely '
                                    'provides read-only access to users.')

    def setUp(self):
        super(TrustsV3TestJSON, self).setUp()
        # Use alt_username as the trustee
        self.trust_id = None
        self.create_trustor_and_roles()

    def tearDown(self):
        if self.trust_id:
            # Do the delete in tearDown not addCleanup - we want the test to
            # fail in the event there is a bug which causes undeletable trusts
            self.delete_trust()
        super(TrustsV3TestJSON, self).tearDown()

    def create_trustor_and_roles(self):
        prefix = CONF.resource_name_prefix
        # create a project that trusts will be granted on
        trustor_project_name = data_utils.rand_name(
            name=self.__class__.__name__, prefix=prefix)
        project = self.projects_client.create_project(
            trustor_project_name,
            domain_id=CONF.identity.default_domain_id)['project']
        self.trustor_project_id = project['id']
        self.addCleanup(self.projects_client.delete_project, project['id'])
        self.assertIsNotNone(self.trustor_project_id)

        # Create a trustor User
        trustor_username = data_utils.rand_name(name='user', prefix=prefix)
        u_desc = trustor_username + 'description'
        u_email = trustor_username + '@testmail.xx'
        trustor_password = data_utils.rand_password()
        user = self.users_client.create_user(
            name=trustor_username,
            description=u_desc,
            password=trustor_password,
            email=u_email,
            project_id=self.trustor_project_id,
            domain_id=CONF.identity.default_domain_id)['user']
        self.addCleanup(self.users_client.delete_user, user['id'])
        self.trustor_user_id = user['id']

        # And two roles, one we'll delegate and one we won't
        self.delegated_role = data_utils.rand_name(
            name='DelegatedRole', prefix=prefix)
        self.not_delegated_role = data_utils.rand_name(
            name='NotDelegatedRole', prefix=prefix)

        role = self.roles_client.create_role(name=self.delegated_role)['role']
        self.addCleanup(self.roles_client.delete_role, role['id'])
        self.delegated_role_id = role['id']

        role = self.roles_client.create_role(
            name=self.not_delegated_role)['role']
        self.addCleanup(self.roles_client.delete_role, role['id'])
        self.not_delegated_role_id = role['id']

        # Assign roles to trustor
        self.roles_client.create_user_role_on_project(
            self.trustor_project_id,
            self.trustor_user_id,
            self.delegated_role_id)
        self.roles_client.create_user_role_on_project(
            self.trustor_project_id,
            self.trustor_user_id,
            self.not_delegated_role_id)

        # Get trustee user ID, use the demo user
        trustee_username = self.non_admin_client.user
        self.trustee_user_id = self.get_user_by_name(trustee_username)['id']
        self.assertIsNotNone(self.trustee_user_id)

        # Initialize a new client with the trustor credentials
        creds = common_creds.get_credentials(
            identity_version='v3',
            username=trustor_username,
            password=trustor_password,
            user_domain_id=CONF.identity.default_domain_id,
            tenant_name=trustor_project_name,
            project_domain_id=CONF.identity.default_domain_id,
            domain_id=CONF.identity.default_domain_id)
        os = clients.Manager(credentials=creds)
        self.trustor_client = os.trusts_client

    def create_trust(self, impersonate=True, expires=None):

        trust_create = self.trustor_client.create_trust(
            trustor_user_id=self.trustor_user_id,
            trustee_user_id=self.trustee_user_id,
            project_id=self.trustor_project_id,
            roles=[{'name': self.delegated_role}],
            impersonation=impersonate,
            expires_at=expires)['trust']
        self.trust_id = trust_create['id']
        return trust_create

    def validate_trust(self, trust, impersonate=True, expires=None,
                       summary=False):
        self.assertIsNotNone(trust['id'])
        self.assertEqual(impersonate, trust['impersonation'])
        if expires is not None:
            # Omit microseconds component of the expiry time
            trust_expires_at = re.sub(r'\.([0-9]){6}', '', trust['expires_at'])
            self.assertEqual(expires, trust_expires_at)
        else:
            self.assertIsNone(trust['expires_at'])
        self.assertEqual(self.trustor_user_id, trust['trustor_user_id'])
        self.assertEqual(self.trustee_user_id, trust['trustee_user_id'])
        self.assertIn('v3/OS-TRUST/trusts', trust['links']['self'])
        self.assertEqual(self.trustor_project_id, trust['project_id'])
        if not summary:
            self.assertEqual(self.delegated_role, trust['roles'][0]['name'])
            self.assertEqual(1, len(trust['roles']))

    def show_trust(self):
        trust_get = self.trustor_client.show_trust(self.trust_id)['trust']
        return trust_get

    def validate_role(self, role):
        self.assertEqual(self.delegated_role_id, role['id'])
        self.assertEqual(self.delegated_role, role['name'])
        self.assertIn('v3/roles/%s' % self.delegated_role_id,
                      role['links']['self'])
        self.assertNotEqual(self.not_delegated_role_id, role['id'])
        self.assertNotEqual(self.not_delegated_role, role['name'])
        self.assertNotIn('v3/roles/%s' % self.not_delegated_role_id,
                         role['links']['self'])

    def check_trust_roles(self):
        # Check we find the delegated role
        roles_get = self.trustor_client.list_trust_roles(
            self.trust_id)['roles']
        self.assertEqual(1, len(roles_get))
        self.validate_role(roles_get[0])

        role_get = self.trustor_client.show_trust_role(
            self.trust_id, self.delegated_role_id)['role']
        self.validate_role(role_get)

        role_get = self.trustor_client.check_trust_role(
            self.trust_id, self.delegated_role_id)

        # And that we don't find not_delegated_role
        self.assertRaises(lib_exc.NotFound,
                          self.trustor_client.show_trust_role,
                          self.trust_id,
                          self.not_delegated_role_id)

        self.assertRaises(lib_exc.NotFound,
                          self.trustor_client.check_trust_role,
                          self.trust_id,
                          self.not_delegated_role_id)

    def delete_trust(self):
        self.trustor_client.delete_trust(self.trust_id)
        self.assertRaises(lib_exc.NotFound,
                          self.trustor_client.show_trust,
                          self.trust_id)
        self.trust_id = None

    @decorators.idempotent_id('5a0a91a4-baef-4a14-baba-59bf4d7fcace')
    def test_trust_impersonate(self):
        """Test keystone trust with impersonation enabled

        To check we can create, get and delete a trust.
        Updates are not supported for trusts
        """
        trust = self.create_trust()
        self.validate_trust(trust)

        trust_get = self.show_trust()
        self.validate_trust(trust_get)

        self.check_trust_roles()

    @decorators.idempotent_id('ed2a8779-a7ac-49dc-afd7-30f32f936ed2')
    def test_trust_noimpersonate(self):
        """Test keystone trust with impersonation disabled

        To check we can create, get and delete a trust
        with impersonation=False
        """
        trust = self.create_trust(impersonate=False)
        self.validate_trust(trust, impersonate=False)

        trust_get = self.show_trust()
        self.validate_trust(trust_get, impersonate=False)

        self.check_trust_roles()

    @decorators.idempotent_id('0ed14b66-cefd-4b5c-a964-65759453e292')
    def test_trust_expire(self):
        """Test expire attribute of keystone trust

        To check we can create, get and delete a trust
        with an expiry specified
        """
        expires_at = timeutils.utcnow() + datetime.timedelta(hours=1)
        # NOTE(ylobankov) In some cases the expiry time may be rounded up
        # because of microseconds. In fact, it depends on database and its
        # version. At least MySQL 5.6.16 does this.
        # For example, when creating a trust, we will set the expiry time of
        # the trust to 2015-02-17T17:34:01.907051Z. However, if we make a GET
        # request on the trust, the response will contain the time rounded up
        # to 2015-02-17T17:34:02.000000Z. That is why we set microsecond to
        # 0 when we invoke isoformat to avoid problems with rounding.
        expires_at = expires_at.replace(microsecond=0)
        # NOTE(ekhugen) Python datetime does not support military timezones
        # since we used UTC we'll add the Z so our compare works.
        expires_str = expires_at.isoformat() + 'Z'

        trust = self.create_trust(expires=expires_str)
        self.validate_trust(trust, expires=expires_str)

        trust_get = self.show_trust()

        self.validate_trust(trust_get, expires=expires_str)

        self.check_trust_roles()

    @decorators.idempotent_id('3e48f95d-e660-4fa9-85e0-5a3d85594384')
    def test_trust_expire_invalid(self):
        """Test invalid expire attribute of a keystone trust

        To check an invalid expiry time is rejected with the correct error
        """
        # with an expiry specified
        expires_str = 'bad.123Z'
        self.assertRaises(lib_exc.BadRequest,
                          self.create_trust,
                          expires=expires_str)

    @decorators.idempotent_id('6268b345-87ca-47c0-9ce3-37792b43403a')
    def test_get_trusts_query(self):
        """Test getting keystone trusts"""
        self.create_trust()
        trusts_get = self.trustor_client.list_trusts(
            trustor_user_id=self.trustor_user_id)['trusts']
        self.assertEqual(1, len(trusts_get))
        self.validate_trust(trusts_get[0], summary=True)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('4773ebd5-ecbf-4255-b8d8-b63e6f72b65d')
    def test_get_trusts_all(self):
        """Test getting all keystone trusts"""
        # Simple function that can be used for cleanup
        def set_scope(auth_provider, scope):
            auth_provider.scope = scope

        self.create_trust()
        # Listing trusts can be done by trustor, by trustee, or without
        # any filter if scoped to a project, so we must ensure token scope is
        # project for this test.
        original_scope = self.os_admin.auth_provider.scope
        set_scope(self.os_admin.auth_provider, 'project')
        self.addCleanup(set_scope, self.os_admin.auth_provider, original_scope)
        trusts_get = self.trusts_client.list_trusts()['trusts']
        trusts = [t for t in trusts_get
                  if t['id'] == self.trust_id]
        self.assertEqual(1, len(trusts))
        self.validate_trust(trusts[0], summary=True)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright 2017 AT&T Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import \
    security_groups as security_groups_schema
from tempest.lib.api_schema.response.compute.v2_1 import servers as schema
from tempest.lib.api_schema.response.compute.v2_16 import servers as schemav216
from tempest.lib.api_schema.response.compute.v2_19 import servers as schemav219
from tempest.lib.api_schema.response.compute.v2_26 import servers as schemav226
from tempest.lib.api_schema.response.compute.v2_3 import servers as schemav23
from tempest.lib.api_schema.response.compute.v2_45 import servers as schemav245
from tempest.lib.api_schema.response.compute.v2_47 import servers as schemav247
from tempest.lib.api_schema.response.compute.v2_48 import servers as schemav248
from tempest.lib.api_schema.response.compute.v2_51 import servers as schemav251
from tempest.lib.api_schema.response.compute.v2_54 import servers as schemav254
from tempest.lib.api_schema.response.compute.v2_57 import servers as schemav257
from tempest.lib.api_schema.response.compute.v2_58 import servers as schemav258
from tempest.lib.api_schema.response.compute.v2_6 import servers as schemav26
from tempest.lib.api_schema.response.compute.v2_62 import servers as schemav262
from tempest.lib.api_schema.response.compute.v2_63 import servers as schemav263
from tempest.lib.api_schema.response.compute.v2_70 import servers as schemav270
from tempest.lib.api_schema.response.compute.v2_71 import servers as schemav271
from tempest.lib.api_schema.response.compute.v2_73 import servers as schemav273
from tempest.lib.api_schema.response.compute.v2_75 import servers as schemav275
from tempest.lib.api_schema.response.compute.v2_79 import servers as schemav279
from tempest.lib.api_schema.response.compute.v2_8 import servers as schemav28
from tempest.lib.api_schema.response.compute.v2_89 import servers as schemav289
from tempest.lib.api_schema.response.compute.v2_9 import servers as schemav29
from tempest.lib.api_schema.response.compute.v2_96 import servers as schemav296
from tempest.lib.api_schema.response.compute.v2_98 import servers as schemav298
from tempest.lib.api_schema.response.compute.v2_99 import servers as schemav299
from tempest.lib.api_schema.response.compute.v2_100 import \
    servers as schemav2100  # noqa: H306
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class ServersClient(base_compute_client.BaseComputeClient):
    """Service client for the resource /servers"""

    schema_versions_info = [
        {'min': None, 'max': '2.2', 'schema': schema},
        {'min': '2.3', 'max': '2.5', 'schema': schemav23},
        {'min': '2.6', 'max': '2.7', 'schema': schemav26},
        {'min': '2.8', 'max': '2.8', 'schema': schemav28},
        {'min': '2.9', 'max': '2.15', 'schema': schemav29},
        {'min': '2.16', 'max': '2.18', 'schema': schemav216},
        {'min': '2.19', 'max': '2.25', 'schema': schemav219},
        {'min': '2.26', 'max': '2.44', 'schema': schemav226},
        {'min': '2.45', 'max': '2.46', 'schema': schemav245},
        {'min': '2.47', 'max': '2.47', 'schema': schemav247},
        {'min': '2.48', 'max': '2.50', 'schema': schemav248},
        {'min': '2.51', 'max': '2.53', 'schema': schemav251},
        {'min': '2.54', 'max': '2.56', 'schema': schemav254},
        {'min': '2.57', 'max': '2.57', 'schema': schemav257},
        {'min': '2.58', 'max': '2.61', 'schema': schemav258},
        {'min': '2.62', 'max': '2.62', 'schema': schemav262},
        {'min': '2.63', 'max': '2.69', 'schema': schemav263},
        {'min': '2.70', 'max': '2.70', 'schema': schemav270},
        {'min': '2.71', 'max': '2.72', 'schema': schemav271},
        {'min': '2.73', 'max': '2.74', 'schema': schemav273},
        {'min': '2.75', 'max': '2.78', 'schema': schemav275},
        {'min': '2.79', 'max': '2.88', 'schema': schemav279},
        {'min': '2.89', 'max': '2.95', 'schema': schemav289},
        {'min': '2.96', 'max': '2.97', 'schema': schemav296},
        {'min': '2.98', 'max': '2.98', 'schema': schemav298},
        {'min': '2.99', 'max': '2.99', 'schema': schemav299},
        {'min': '2.100', 'max': None, 'schema': schemav2100},
    ]

    def __init__(self, auth_provider, service, region,
                 enable_instance_password=True, **kwargs):
        super(ServersClient, self).__init__(
            auth_provider, service, region, **kwargs)
        self.enable_instance_password = enable_instance_password

    def create_server(self, **kwargs):
        """Create server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-server

        :param name: Server name
        :param imageRef: Image reference (UUID)
        :param flavorRef: Flavor reference (UUID or full URL)

        Most parameters except the following are passed to the API without
        any changes.
        :param disk_config: The name is changed to OS-DCF:diskConfig
        :param scheduler_hints: The name is changed to os:scheduler_hints and
        the parameter is set in the same level as the parameter 'server'.
        """
        body = copy.deepcopy(kwargs)
        if body.get('disk_config'):
            body['OS-DCF:diskConfig'] = body.pop('disk_config')

        hints = None
        if body.get('scheduler_hints'):
            hints = {'os:scheduler_hints': body.pop('scheduler_hints')}

        post_body = {'server': body}

        if hints:
            post_body.update(hints)

        post_body = json.dumps(post_body)
        resp, body = self.post('servers', post_body)

        body = json.loads(body)
        # NOTE(maurosr): this deals with the case of multiple server create
        # with return reservation id set True
        if 'reservation_id' in body:
            return rest_client.ResponseBody(resp, body)
        if self.enable_instance_password:
            create_schema = schema.create_server_with_admin_pass
        else:
            create_schema = schema.create_server
        self.validate_response(create_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_server(self, server_id, **kwargs):
        """Update server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-server

        Most parameters except the following are passed to the API without
        any changes.
        :param disk_config: The name is changed to OS-DCF:diskConfig
        """
        if 'disk_config' in kwargs:
            kwargs['OS-DCF:diskConfig'] = kwargs.pop('disk_config')

        post_body = json.dumps({'server': kwargs})
        resp, body = self.put("servers/%s" % server_id, post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.update_server, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_server(self, server_id):
        """Get server details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-server-details
        """
        resp, body = self.get("servers/%s" % server_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_server, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_server(self, server_id):
        """Delete server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-server
        """
        resp, body = self.delete("servers/%s" % server_id)
        self.validate_response(schema.delete_server, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_servers(self, detail=False, **params):
        """List servers.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-servers
        https://docs.openstack.org/api-ref/compute/#list-servers-detailed
        """

        url = 'servers'
        schema = self.get_schema(self.schema_versions_info)
        if detail:
            url += '/detail'
            _schema = schema.list_servers_detail
        else:
            _schema = schema.list_servers
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(_schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_addresses(self, server_id):
        """Lists all addresses for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-ips
        """
        resp, body = self.get("servers/%s/ips" % server_id)
        body = json.loads(body)
        self.validate_response(schema.list_addresses, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_addresses_by_network(self, server_id, network_id):
        """Lists all addresses of a specific network type for a server."""
        resp, body = self.get("servers/%s/ips/%s" %
                              (server_id, network_id))
        body = json.loads(body)
        self.validate_response(schema.list_addresses_by_network, resp, body)
        return rest_client.ResponseBody(resp, body)

    def action(self, server_id, action_name,
               schema=schema.server_actions_common_schema,
               **kwargs):
        if 'body' in kwargs:
            post_body = json.dumps(kwargs['body'])
        else:
            post_body = json.dumps({action_name: kwargs})
        resp, body = self.post('servers/%s/action' % server_id,
                               post_body)
        if body:
            body = json.loads(body)
        else:
            if isinstance(body, bytes):
                body = body.decode('utf-8')
        self.validate_response(schema, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_backup(self, server_id, **kwargs):
        """Backup a server instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-server-back-up-createbackup-action
        """
        schema = self.get_schema(self.schema_versions_info)
        return self.action(server_id, "createBackup",
                           schema.create_backup, **kwargs)

    def change_password(self, server_id, **kwargs):
        """Change the root password for the server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#change-administrative-password-changepassword-action
        """
        return self.action(server_id, 'changePassword', **kwargs)

    def show_password(self, server_id):
        resp, body = self.get("servers/%s/os-server-password" %
                              server_id)
        body = json.loads(body)
        self.validate_response(schema.show_password, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_password(self, server_id):
        """Removes the encrypted server password from the metadata server

        Note that this does not actually change the instance server
        password.
        """
        resp, body = self.delete("servers/%s/os-server-password" %
                                 server_id)
        self.validate_response(schema.server_actions_delete_password,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def reboot_server(self, server_id, **kwargs):
        """Reboot a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#reboot-server-reboot-action
        """
        return self.action(server_id, 'reboot', **kwargs)

    def rebuild_server(self, server_id, image_ref, **kwargs):
        """Rebuild a server with a new image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#rebuild-server-rebuild-action

        Most parameters except the following are passed to the API without
        any changes.
        :param disk_config: The name is changed to OS-DCF:diskConfig
        """
        kwargs['imageRef'] = image_ref
        if 'disk_config' in kwargs:
            kwargs['OS-DCF:diskConfig'] = kwargs.pop('disk_config')
        schema = self.get_schema(self.schema_versions_info)
        if self.enable_instance_password:
            rebuild_schema = schema.rebuild_server_with_admin_pass
        else:
            rebuild_schema = schema.rebuild_server
        return self.action(server_id, 'rebuild',
                           rebuild_schema, **kwargs)

    def resize_server(self, server_id, flavor_ref, **kwargs):
        """Change the flavor of a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#resize-server-resize-action

        Most parameters except the following are passed to the API without
        any changes.
        :param disk_config: The name is changed to OS-DCF:diskConfig
        """
        kwargs['flavorRef'] = flavor_ref
        if 'disk_config' in kwargs:
            kwargs['OS-DCF:diskConfig'] = kwargs.pop('disk_config')
        return self.action(server_id, 'resize', **kwargs)

    def confirm_resize_server(self, server_id, **kwargs):
        """Confirm the flavor change for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#confirm-resized-server-confirmresize-action
        """
        return self.action(server_id, 'confirmResize',
                           schema.server_actions_confirm_resize,
                           **kwargs)

    def revert_resize_server(self, server_id, **kwargs):
        """Revert a server back to its original flavor.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#revert-resized-server-revertresize-action
        """
        return self.action(server_id, 'revertResize', **kwargs)

    def list_server_metadata(self, server_id):
        """Lists all metadata for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-all-metadata
        """
        resp, body = self.get("servers/%s/metadata" % server_id)
        body = json.loads(body)
        self.validate_response(schema.list_server_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_server_metadata(self, server_id, meta, no_metadata_field=False):
        """Sets one or more metadata items for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#replace-metadata-items
        """
        if no_metadata_field:
            post_body = ""
        else:
            post_body = json.dumps({'metadata': meta})
        resp, body = self.put('servers/%s/metadata' % server_id,
                              post_body)
        body = json.loads(body)
        self.validate_response(schema.set_server_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_server_metadata(self, server_id, meta):
        """Updates one or more metadata items for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-update-metadata-items
        """
        post_body = json.dumps({'metadata': meta})
        resp, body = self.post('servers/%s/metadata' % server_id,
                               post_body)
        body = json.loads(body)
        self.validate_response(schema.update_server_metadata,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_server_metadata_item(self, server_id, key):
        """Shows details for a metadata item, by key, for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-metadata-item-details
        """
        resp, body = self.get("servers/%s/metadata/%s" % (server_id, key))
        body = json.loads(body)
        self.validate_response(schema.set_show_server_metadata_item,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_server_metadata_item(self, server_id, key, meta):
        """Sets a metadata item, by key, for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-update-metadata-item
        """
        post_body = json.dumps({'meta': meta})
        resp, body = self.put('servers/%s/metadata/%s' % (server_id, key),
                              post_body)
        body = json.loads(body)
        self.validate_response(schema.set_show_server_metadata_item,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_server_metadata_item(self, server_id, key):
        """Deletes a metadata item, by key, from a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-metadata-item
        """
        resp, body = self.delete("servers/%s/metadata/%s" %
                                 (server_id, key))
        self.validate_response(schema.delete_server_metadata_item,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def stop_server(self, server_id, **kwargs):
        """Stops a running server and changes its status to SHUTOFF.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#stop-server-os-stop-action
        """
        return self.action(server_id, 'os-stop', **kwargs)

    def start_server(self, server_id, **kwargs):
        """Starts a stopped server and changes its status to ACTIVE.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#start-server-os-start-action
        """
        return self.action(server_id, 'os-start', **kwargs)

    def attach_volume(self, server_id, **kwargs):
        """Attaches a volume to a server instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#attach-a-volume-to-an-instance
        """
        post_body = json.dumps({'volumeAttachment': kwargs})
        resp, body = self.post('servers/%s/os-volume_attachments' % server_id,
                               post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.attach_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_attached_volume(self, server_id, attachment_id, **kwargs):
        """Swaps a volume attached to an instance for another volume

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-a-volume-attachment
        """
        post_body = json.dumps({'volumeAttachment': kwargs})
        resp, body = self.put('servers/%s/os-volume_attachments/%s' %
                              (server_id, attachment_id),
                              post_body)
        self.validate_response(schema.update_attached_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def detach_volume(self, server_id, volume_id):  # noqa
        """Detaches a volume from a server instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#detach-a-volume-from-an-instance
        """
        resp, body = self.delete('servers/%s/os-volume_attachments/%s' %
                                 (server_id, volume_id))
        self.validate_response(schema.detach_volume, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_volume_attachment(self, server_id, volume_id):
        """Return details about the given volume attachment.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-a-detail-of-a-volume-attachment
        """
        resp, body = self.get('servers/%s/os-volume_attachments/%s' % (
            server_id, volume_id))
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_volume_attachment, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_volume_attachments(self, server_id):
        """Returns the list of volume attachments for a given instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-volume-attachments-for-an-instance
        """
        resp, body = self.get('servers/%s/os-volume_attachments' % (
            server_id))
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_volume_attachments, resp, body)
        return rest_client.ResponseBody(resp, body)

    def add_security_group(self, server_id, **kwargs):
        """Add a security group to the server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#add-security-group-to-a-server-addsecuritygroup-action
        """
        return self.action(server_id, 'addSecurityGroup', **kwargs)

    def remove_security_group(self, server_id, **kwargs):
        """Remove a security group from the server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#remove-security-group-from-a-server-removesecuritygroup-action
        """
        return self.action(server_id, 'removeSecurityGroup', **kwargs)

    def live_migrate_server(self, server_id, **kwargs):
        """This should be called with administrator privileges.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#live-migrate-server-os-migratelive-action
        """
        return self.action(server_id, 'os-migrateLive', **kwargs)

    def migrate_server(self, server_id, **kwargs):
        """Migrate a server to a new host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#migrate-server-migrate-action
        """
        return self.action(server_id, 'migrate', **kwargs)

    def lock_server(self, server_id, **kwargs):
        """Lock the given server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#lock-server-lock-action
        """
        return self.action(server_id, 'lock', **kwargs)

    def unlock_server(self, server_id, **kwargs):
        """UNlock the given server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#unlock-server-unlock-action
        """
        return self.action(server_id, 'unlock', **kwargs)

    def suspend_server(self, server_id, **kwargs):
        """Suspend the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#suspend-server-suspend-action
        """
        return self.action(server_id, 'suspend', **kwargs)

    def resume_server(self, server_id, **kwargs):
        """Un-suspend the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#resume-suspended-server-resume-action
        """
        return self.action(server_id, 'resume', **kwargs)

    def pause_server(self, server_id, **kwargs):
        """Pause the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#pause-server-pause-action
        """
        return self.action(server_id, 'pause', **kwargs)

    def unpause_server(self, server_id, **kwargs):
        """Un-pause the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#unpause-server-unpause-action
        """
        return self.action(server_id, 'unpause', **kwargs)

    def reset_state(self, server_id, **kwargs):
        """Reset the state of a server to active/error.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#reset-server-state-os-resetstate-action
        """
        return self.action(server_id, 'os-resetState', **kwargs)

    def shelve_server(self, server_id, **kwargs):
        """Shelve the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#shelve-server-shelve-action
        """
        return self.action(server_id, 'shelve', **kwargs)

    def unshelve_server(self, server_id, **kwargs):
        """Un-shelve the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#unshelve-restore-shelved-server-unshelve-action
        """
        # NOTE(gmann): pass None as request body if nothing is requested.
        # Nova started the request body check since 2.77 microversion and only
        # accept AZ or None as valid req body and reject the empty dict {}.
        # Before 2.77 microverison anything is valid body as Nova does not
        # check the request body but as per api-ref None is valid request
        # body to pass so we do not need to check the requested microversion
        # here and always default req body to None.
        if not kwargs:
            kwargs['body'] = {'unshelve': None}
        return self.action(server_id, 'unshelve', **kwargs)

    def shelve_offload_server(self, server_id, **kwargs):
        """Shelve-offload the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#shelf-offload-remove-server-shelveoffload-action
        """
        return self.action(server_id, 'shelveOffload', **kwargs)

    def get_console_output(self, server_id, **kwargs):
        """Get console output.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-console-output-os-getconsoleoutput-action
        """
        return self.action(server_id, 'os-getConsoleOutput',
                           schema.get_console_output, **kwargs)

    def get_remote_console(self, server_id, console_type, protocol, **kwargs):
        """Get a remote console.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-console
        """
        param = {
            'remote_console': {
                'type': console_type,
                'protocol': protocol,
            }
        }
        post_body = json.dumps(param)
        resp, body = self.post("servers/%s/remote-consoles" % server_id,
                               post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_remote_consoles, resp, body)
        return rest_client.ResponseBody(resp, body)

    def get_console_auth_token_details(self, token):
        """Turn a console auth token into hypervisor connection details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-console-connection-information
        """
        resp, body = self.get('/os-console-auth-tokens/%s' % token)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.console_auth_tokens, resp, body)
        return rest_client.ResponseBody(resp, body)

    def rescue_server(self, server_id, **kwargs):
        """Rescue the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#rescue-server-rescue-action
        """
        if self.enable_instance_password:
            rescue_schema = schema.rescue_server_with_admin_pass
        else:
            rescue_schema = schema.rescue_server
        return self.action(server_id, 'rescue', rescue_schema, **kwargs)

    def unrescue_server(self, server_id):
        """Unrescue the provided server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#unrescue-server-unrescue-action
        """
        return self.action(server_id, 'unrescue')

    def show_server_diagnostics(self, server_id):
        """Get the usage data for a server."""
        resp, body = self.get("servers/%s/diagnostics" % server_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_server_diagnostics, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_instance_actions(self, server_id):
        """List the provided server action."""
        resp, body = self.get("servers/%s/os-instance-actions" %
                              server_id)
        body = json.loads(body)
        self.validate_response(schema.list_instance_actions, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_instance_action(self, server_id, request_id):
        """Returns the action details of the provided server."""
        resp, body = self.get("servers/%s/os-instance-actions/%s" %
                              (server_id, request_id))
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.show_instance_action, resp, body)
        return rest_client.ResponseBody(resp, body)

    def force_delete_server(self, server_id, **kwargs):
        """Force delete a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#force-delete-server-forcedelete-action
        """
        return self.action(server_id, 'forceDelete', **kwargs)

    def restore_soft_deleted_server(self, server_id, **kwargs):
        """Restore a soft-deleted server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#restore-soft-deleted-instance-restore-action
        """
        return self.action(server_id, 'restore', **kwargs)

    def reset_network(self, server_id, **kwargs):
        """Reset the Network of a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#reset-networking-on-a-server-resetnetwork-action
        """
        return self.action(server_id, 'resetNetwork', **kwargs)

    def inject_network_info(self, server_id, **kwargs):
        """Inject the Network Info into server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#inject-network-information-injectnetworkinfo-action
        """
        return self.action(server_id, 'injectNetworkInfo', **kwargs)

    def get_vnc_console(self, server_id, **kwargs):
        """Get URL of VNC console.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#get-vnc-console-os-getvncconsole-action-deprecated
        """
        return self.action(server_id, "os-getVNCConsole",
                           schema.get_vnc_console, **kwargs)

    def add_fixed_ip(self, server_id, **kwargs):
        """Add a fixed IP to server instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#add-associate-fixed-ip-addfixedip-action-deprecated
        """
        return self.action(server_id, 'addFixedIp', **kwargs)

    def remove_fixed_ip(self, server_id, **kwargs):
        """Remove input fixed IP from input server instance.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#remove-disassociate-fixed-ip-removefixedip-action-deprecated
        """
        return self.action(server_id, 'removeFixedIp', **kwargs)

    def list_security_groups_by_server(self, server_id):
        """Lists security groups for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-security-groups-by-server
        """
        resp, body = self.get("servers/%s/os-security-groups" % server_id)
        body = json.loads(body)
        self.validate_response(security_groups_schema.list_security_groups,
                               resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_tags(self, server_id):
        """Lists all tags for a server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-tags
        """
        url = 'servers/%s/tags' % server_id
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_tags, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_all_tags(self, server_id, tags):
        """Replaces all tags on specified server with the new set of tags.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#replace-tags

        :param tags: List of tags to replace current server tags with.
        """
        url = 'servers/%s/tags' % server_id
        put_body = {'tags': tags}
        resp, body = self.put(url, json.dumps(put_body))
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.update_all_tags, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_all_tags(self, server_id):
        """Deletes all tags from the specified server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-all-tags
        """
        url = 'servers/%s/tags' % server_id
        resp, body = self.delete(url)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_all_tags, resp, body)
        return rest_client.ResponseBody(resp, body)

    def check_tag_existence(self, server_id, tag):
        """Checks tag existence on the server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#check-tag-existence

        :param tag: Check for existence of tag on specified server.
        """
        url = 'servers/%s/tags/%s' % (server_id, tag)
        resp, body = self.get(url)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.check_tag_existence, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_tag(self, server_id, tag):
        """Adds a single tag to the server if server has no specified tag.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#add-a-single-tag

        :param tag: Tag to be added to the specified server.
        """
        url = 'servers/%s/tags/%s' % (server_id, tag)
        resp, body = self.put(url, None)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.update_tag, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_tag(self, server_id, tag):
        """Deletes a single tag from the specified server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#delete-a-single-tag

        :param tag: Tag to be removed from the specified server.
        """
        url = 'servers/%s/tags/%s' % (server_id, tag)
        resp, body = self.delete(url)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_tag, resp, body)
        return rest_client.ResponseBody(resp, body)

    def evacuate_server(self, server_id, **kwargs):
        """Evacuate the given server.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#evacuate-server-evacuate-action
        """
        api_version = self.get_headers().get(self.api_microversion_header_name)

        if not api_version and self.enable_instance_password:
            evacuate_schema = schema.evacuate_server_with_admin_pass
        elif api_version < '2.14':
            evacuate_schema = schema.evacuate_server_with_admin_pass
        else:
            evacuate_schema = schema.evacuate_server

        return self.action(server_id, 'evacuate',
                           evacuate_schema,
                           **kwargs)



# Copyright 2022 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import signal
import subprocess

import fixtures

from oslo_log import log


LOG = log.getLogger(__name__)

PASSED = 'PASSED'
FAILED = 'FAILED'
METADATA_SCRIPT_PATH = '/tmp/metadata_meter_script.sh'
METADATA_RESULTS_PATH = '/tmp/metadata_meter.log'
METADATA_PID_PATH = '/tmp/metadata_meter.pid'
# /proc/uptime is used because it include two decimals in cirros, while
# `date +%s.%N` does not work in cirros (min granularity is seconds)
METADATA_SCRIPT = """#!/bin/sh
echo $$ > %(metadata_meter_pidfile)s
old_time=$(cut -d" " -f1 /proc/uptime)
while true; do
    curl http://169.254.169.254/latest/meta-data/hostname 2>/dev/null | \
grep -q `hostname`
    result=$?
    new_time=$(cut -d" " -f1 /proc/uptime)
    runtime=$(awk -v new=$new_time -v old=$old_time "BEGIN {print new-old}")
    old_time=$new_time
    if [ $result -eq 0 ]; then
        echo "PASSED $runtime"
    else
        echo "FAILED $runtime"
    fi
    sleep %(interval)s
done
"""


class NetDowntimeMeter(fixtures.Fixture):
    def __init__(self, dest_ip, interval=0.2):
        self.dest_ip = dest_ip
        # Note: for intervals lower than 0.2 ping requires root privileges
        self.interval = float(interval)
        self.ping_process = None

    def _setUp(self):
        self.start_background_pinger()

    def start_background_pinger(self):
        cmd = ['ping', '-q', '-s1']
        cmd.append('-i%g' % self.interval)
        cmd.append(self.dest_ip)
        LOG.debug("Starting background pinger to '%s' with interval %g",
                  self.dest_ip, self.interval)
        self.ping_process = subprocess.Popen(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        self.addCleanup(self.cleanup)

    def cleanup(self):
        if self.ping_process and self.ping_process.poll() is None:
            LOG.debug('Terminating background pinger with pid %d',
                      self.ping_process.pid)
            self.ping_process.terminate()
        self.ping_process = None

    def get_downtime(self):
        self.ping_process.send_signal(signal.SIGQUIT)
        # Example of the expected output:
        # 264/274 packets, 3% loss
        output = self.ping_process.stderr.readline().strip().decode('utf-8')
        if output and len(output.split()[0].split('/')) == 2:
            succ, total = output.split()[0].split('/')
            return (int(total) - int(succ)) * self.interval
        else:
            LOG.warning('Unexpected output obtained from the pinger: %s',
                        output)


class MetadataDowntimeMeter(fixtures.Fixture):
    def __init__(self, ssh_client,
                 interval='0.2', script_path=METADATA_SCRIPT_PATH,
                 output_path=METADATA_RESULTS_PATH,
                 pidfile_path=METADATA_PID_PATH):
        self.ssh_client = ssh_client
        self.interval = interval
        self.script_path = script_path
        self.output_path = output_path
        self.pidfile_path = pidfile_path
        self.pid = None

    def _setUp(self):
        self.addCleanup(self.cleanup)
        self.upload_metadata_script()
        self.run_metadata_script()

    def upload_metadata_script(self):
        metadata_script = METADATA_SCRIPT % {
            'metadata_meter_pidfile': self.pidfile_path,
            'interval': self.interval}
        echo_cmd = "echo '{}' > {}".format(
            metadata_script, self.script_path)
        chmod_cmd = 'chmod +x {}'.format(self.script_path)
        self.ssh_client.exec_command(';'.join((echo_cmd, chmod_cmd)))
        LOG.debug('script created: %s', self.script_path)
        output = self.ssh_client.exec_command(
            'cat {}'.format(self.script_path))
        LOG.debug('script content: %s', output)

    def run_metadata_script(self):
        self.ssh_client.exec_command('{} > {} &'.format(self.script_path,
                                                        self.output_path))
        self.pid = self.ssh_client.exec_command(
            'cat {}'.format(self.pidfile_path)).strip()
        LOG.debug('running metadata downtime meter script in background with '
                  'PID = %s', self.pid)

    def get_results(self):
        output = self.ssh_client.exec_command(
            'cat {}'.format(self.output_path))
        results = {}
        results['successes'] = output.count(PASSED)
        results['failures'] = output.count(FAILED)
        downtime = {PASSED: 0.0, FAILED: 0.0}
        for line in output.splitlines():
            key, value = line.strip().split()
            downtime[key] += float(value)

        results['downtime'] = downtime
        LOG.debug('metadata downtime meter results: %r', results)
        return results

    def cleanup(self):
        if self.pid:
            self.ssh_client.exec_command('kill {}'.format(self.pid))
            LOG.debug('killed metadata downtime script with PID %s', self.pid)
        else:
            LOG.debug('No metadata downtime script found')



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class VolumesTransfersTest(base.BaseVolumeTest):
    """Test volume transfer"""

    credentials = ['primary', 'alt', 'admin']

    @classmethod
    def setup_clients(cls):
        super(VolumesTransfersTest, cls).setup_clients()

        cls.client = cls.os_primary.volume_transfers_client_latest
        cls.alt_client = cls.os_alt.volume_transfers_client_latest
        cls.alt_volumes_client = cls.os_alt.volumes_client_latest
        cls.adm_volumes_client = cls.os_admin.volumes_client_latest

    @decorators.idempotent_id('4d75b645-a478-48b1-97c8-503f64242f1a')
    def test_create_get_list_accept_volume_transfer(self):
        """Test creating, getting, listing and accepting of volume transfer"""
        # Create a volume first
        volume = self.create_volume()
        self.addCleanup(self.delete_volume,
                        self.adm_volumes_client,
                        volume['id'])

        # Create a volume transfer
        transfer = self.client.create_volume_transfer(
            volume_id=volume['id'])['transfer']
        transfer_id = transfer['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_volume_transfer,
                        transfer_id)
        auth_key = transfer['auth_key']
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume['id'], 'awaiting-transfer')

        # Get a volume transfer
        body = self.client.show_volume_transfer(transfer_id)['transfer']
        self.assertEqual(volume['id'], body['volume_id'])

        # List volume transfers, the result should be greater than
        # or equal to 1
        body = self.client.list_volume_transfers()['transfers']
        self.assertNotEmpty(body)

        # Accept a volume transfer by alt_tenant
        body = self.alt_client.accept_volume_transfer(
            transfer_id, auth_key=auth_key)['transfer']
        waiters.wait_for_volume_resource_status(self.alt_volumes_client,
                                                volume['id'], 'available')
        accepted_volume = self.alt_volumes_client.show_volume(
            volume['id'])['volume']
        self.assertEqual(self.os_alt.credentials.user_id,
                         accepted_volume['user_id'])
        self.assertEqual(self.os_alt.credentials.project_id,
                         accepted_volume['os-vol-tenant-attr:tenant_id'])

    @decorators.idempotent_id('ab526943-b725-4c07-b875-8e8ef87a2c30')
    def test_create_list_delete_volume_transfer(self):
        """Test creating, listing and deleting volume transfer"""
        # Create a volume first
        volume = self.create_volume()
        self.addCleanup(self.delete_volume,
                        self.adm_volumes_client,
                        volume['id'])

        # Create a volume transfer
        transfer_id = self.client.create_volume_transfer(
            volume_id=volume['id'])['transfer']['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_volume_transfer,
                        transfer_id)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume['id'], 'awaiting-transfer')

        # List all volume transfers with details, check the detail-specific
        # elements, and look for the created transfer.
        transfers = self.client.list_volume_transfers(detail=True)['transfers']
        self.assertNotEmpty(transfers)
        volume_list = [transfer['volume_id'] for transfer in transfers]
        self.assertIn(volume['id'], volume_list,
                      'Transfer not found for volume %s' % volume['id'])

        # Delete a volume transfer
        self.client.delete_volume_transfer(transfer_id)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume['id'], 'available')


class VolumesTransfersV355Test(VolumesTransfersTest):
    """Test volume transfer for the "new" Transfers API mv 3.55"""

    volume_min_microversion = '3.55'
    volume_max_microversion = '3.56'

    @classmethod
    def setup_clients(cls):
        super(VolumesTransfersV355Test, cls).setup_clients()
        cls.client = cls.os_primary.volume_transfers_mv355_client_latest
        cls.alt_client = cls.os_alt.volume_transfers_mv355_client_latest

    @decorators.idempotent_id('9f36bb2b-619f-4507-b246-76aeb9a28851')
    def test_create_get_list_accept_volume_transfer(self):
        """Test create, get, list, accept with volume-transfers API mv 3.55"""
        super(VolumesTransfersV355Test, self). \
            test_create_get_list_accept_volume_transfer()

    @decorators.idempotent_id('af4a5b97-0859-4f31-aa3c-85b05bb63322')
    def test_create_list_delete_volume_transfer(self):
        """Test create, list, delete with volume-transfers API mv 3.55"""
        super(VolumesTransfersV355Test, self). \
            test_create_list_delete_volume_transfer()


class VolumesTransfersV357Test(VolumesTransfersV355Test):
    """Test volume transfer for the "new" Transfers API mv 3.57"""

    volume_min_microversion = '3.57'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('d746bd69-bb30-4414-9a1c-577959fac6a1')
    def test_create_get_list_accept_volume_transfer(self):
        """Test create, get, list, accept with volume-transfers API mv 3.57"""
        super(VolumesTransfersV357Test, self). \
            test_create_get_list_accept_volume_transfer()

    @decorators.idempotent_id('d4b20ec2-e1bb-4068-adcf-6c20020a8e05')
    def test_create_list_delete_volume_transfer(self):
        """Test create, list, delete with volume-transfers API mv 3.57"""
        super(VolumesTransfersV357Test, self). \
            test_create_list_delete_volume_transfer()



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class IdentityV3ProjectTagsTest(base.BaseIdentityV3AdminTest):
    """Test keystone project tags"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @decorators.idempotent_id('7c123aac-999d-416a-a0fb-84b915ab10de')
    def test_list_update_delete_project_tags(self):
        """Test listing, updating and deleting of project tags"""
        project = self.setup_test_project()

        # Create a tag for testing.
        tag = data_utils.rand_name('tag', prefix=CONF.resource_name_prefix)
        # NOTE(felipemonteiro): The response body for create is empty.
        self.project_tags_client.update_project_tag(project['id'], tag)

        # Verify that the tag was created.
        self.project_tags_client.check_project_tag_existence(
            project['id'], tag)

        # Verify that updating the project tags works.
        tags_to_update = [data_utils.rand_name(
            'tag', prefix=CONF.resource_name_prefix) for _ in range(3)]
        updated_tags = self.project_tags_client.update_all_project_tags(
            project['id'], tags_to_update)['tags']
        self.assertEqual(sorted(tags_to_update), sorted(updated_tags))

        # Verify that listing project tags works.
        retrieved_tags = self.project_tags_client.list_project_tags(
            project['id'])['tags']
        self.assertEqual(sorted(tags_to_update), sorted(retrieved_tags))

        # Verify that deleting a project tag works.
        self.project_tags_client.delete_project_tag(
            project['id'], tags_to_update[0])
        self.assertRaises(lib_exc.NotFound,
                          self.project_tags_client.check_project_tag_existence,
                          project['id'], tags_to_update[0])

        # Verify that deleting all project tags works.
        self.project_tags_client.delete_all_project_tags(project['id'])
        retrieved_tags = self.project_tags_client.list_project_tags(
            project['id'])['tags']
        self.assertEmpty(retrieved_tags)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute.floating_ips import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class FloatingIPsNegativeTestJSON(base.BaseFloatingIPsTest):
    """Test floating ips API with compute microversion less than 2.36"""

    max_microversion = '2.35'

    @classmethod
    def resource_setup(cls):
        super(FloatingIPsNegativeTestJSON, cls).resource_setup()

        # Generating a nonexistent floatingIP id
        body = cls.client.list_floating_ips()['floating_ips']
        floating_ip_ids = [floating_ip['id'] for floating_ip in body]
        while True:
            if CONF.service_available.neutron:
                cls.non_exist_id = data_utils.rand_uuid()
            else:
                cls.non_exist_id = data_utils.rand_int_id(start=999)
            if cls.non_exist_id not in floating_ip_ids:
                break

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6e0f059b-e4dd-48fb-8207-06e3bba5b074')
    def test_allocate_floating_ip_from_nonexistent_pool(self):
        """Test allocating floating ip from non existent pool should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.create_floating_ip,
                          pool="non_exist_pool")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ae1c55a8-552b-44d4-bfb6-2a115a15d0ba')
    def test_delete_nonexistent_floating_ip(self):
        """Test deleting non existent floating ip should fail"""
        # Deleting the non existent floating IP
        self.assertRaises(lib_exc.NotFound, self.client.delete_floating_ip,
                          self.non_exist_id)


class FloatingIPsAssociationNegativeTestJSON(base.BaseFloatingIPsTest):
    """Test floating ips API with compute microversion less than 2.44"""

    max_microversion = '2.43'

    @classmethod
    def resource_setup(cls):
        super(FloatingIPsAssociationNegativeTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')
        cls.server_id = cls.server['id']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('595fa616-1a71-4670-9614-46564ac49a4c')
    def test_associate_nonexistent_floating_ip(self):
        """Test associating non existent floating ip to server should fail"""
        # Associating non existent floating IP
        self.assertRaises(lib_exc.NotFound,
                          self.client.associate_floating_ip_to_server,
                          "0.0.0.0", self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0a081a66-e568-4e6b-aa62-9587a876dca8')
    def test_dissociate_nonexistent_floating_ip(self):
        """Test dissociating non existent floating ip should fail"""
        # Dissociating non existent floating IP
        self.assertRaises(lib_exc.NotFound,
                          self.client.disassociate_floating_ip_from_server,
                          "0.0.0.0", self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('804b4fcb-bbf5-412f-925d-896672b61eb3')
    def test_associate_ip_to_server_without_passing_floating_ip(self):
        """Test associating empty floating ip to server should fail"""
        # should raise NotFound or BadRequest(In case of Nova V2.1) exception.
        self.assertRaises((lib_exc.NotFound, lib_exc.BadRequest),
                          self.client.associate_floating_ip_to_server,
                          '', self.server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('58a80596-ffb2-11e6-9393-fa163e4fa634')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    def test_associate_ip_to_server_with_floating_ip(self):
        """Test associating floating ip to server already with floating ip

        1. The VM have one port
        2. Associate floating IP A to the VM
        3. Associate floating IP B which is from same pool with floating IP A
           to the VM, should raise BadRequest exception
        """
        body = self.client.create_floating_ip(
            pool=CONF.network.public_network_id)['floating_ip']
        self.addCleanup(self.client.delete_floating_ip, body['id'])
        self.client.associate_floating_ip_to_server(body['ip'], self.server_id)
        self.addCleanup(self.client.disassociate_floating_ip_from_server,
                        body['ip'], self.server_id)

        body = self.client.create_floating_ip(
            pool=CONF.network.public_network_id)['floating_ip']
        self.addCleanup(self.client.delete_floating_ip, body['id'])
        self.assertRaises(lib_exc.BadRequest,
                          self.client.associate_floating_ip_to_server,
                          body['ip'], self.server_id)
